@@ -9,19 +9,70 @@
 package TBX::Min::TermGroup;
 use strict;
 use warnings;
-use parent 'Class::Accessor';
-TBX::Min::TermGroup->mk_accessors(qw(
+use subs qw(part_of_speech status);
+use Class::Tiny qw(
     term
     part_of_speech
     note
     customer
     status
-));
-our $VERSION = '0.03'; # VERSION
+);
+use Carp;
+
+our $VERSION = '0.04'; # VERSION
 
 # ABSTRACT: Store information from one TBX-Min C<termGroup> element
 
 
+sub part_of_speech {
+    my ($self, $pos) = @_;
+    if(defined $pos){
+        _validate_pos($pos);
+        $self->{part_of_speech} = $pos;
+    }
+    return $self->{part_of_speech};
+}
+
+
+sub status {
+    my ($self, $status) = @_;
+    if(defined $status){
+        _validate_status($status);
+        $self->{status} = $status;
+    }
+    return $self->{status};
+}
+
+# above Pod::Coverage makes this not "naked" via Pod::Coverage::TrustPod
+sub BUILD {
+    my ($self, $args) = @_;
+    if($args->{part_of_speech}){
+        _validate_pos($args->{part_of_speech});
+    }
+    if($args->{status}){
+        _validate_status($args->{status});
+    }
+    return;
+}
+
+my @allowed_pos = qw(noun properNoun verb adjective adverb other);
+sub _validate_pos {
+    my ($pos) = @_;
+    if(!grep{$pos eq $_} @allowed_pos){
+        croak "Illegal part of speech '$pos'";
+    }
+    return;
+}
+
+my @allowed_status = qw(admitted preferred notRecommended obsolete);
+sub _validate_status {
+    my ($pos) = @_;
+    if(!grep{$pos eq $_} @allowed_status){
+        croak "Illegal status '$pos'";
+    }
+    return;
+}
+
 1;
 
 __END__
@@ -34,7 +85,7 @@ TBX::Min::TermGroup - Store information from one TBX-Min C<termGroup> element
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 SYNOPSIS
 
@@ -83,6 +134,8 @@ Get or set a status  associated with this term group.
 
 L<TBX::Min>
 
+=for Pod::Coverage BUILD
+
 =head1 AUTHOR
 
 Nathan Glenn <garfieldnate@gmail.com>
